@@ -3,6 +3,7 @@
 /*
  *	stlload.c  -- stallion intelligent multiport down loader.
  *
+ *	Copyright (C) 1996-1997  Stallion Technologies (support@stallion.oz.au).
  *	Copyright (C) 1994-1996  Greg Ungerer (gerg@stallion.oz.au).
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -34,7 +35,7 @@
 
 /*****************************************************************************/
 
-char	*version = "1.1.2";
+char	*version = "5.4.0";
 char	*defdevice = "/dev/staliomem%d";
 char	*image = "/usr/lib/stallion/cdk.sys";
 char	*oldimage = "/usr/lib/stallion/2681.sys";
@@ -135,12 +136,9 @@ int ecpfindports(cdkecpsig_t *sigp)
 			break;
 		if ((id & 0x07) != bank)
 			break;
-		if (id & 0x20) {
-			nrports += 16;
+		nrports += ((id & 0x20) ? 16 : 8);
+		if ((id & 0x20) && ((id & 0x40) == 0))
 			bank++;
-		} else {
-			nrports += 8;
-		}
 	}
 
 	return(nrports);
@@ -190,7 +188,8 @@ int download()
 	if (verbose)
 		printf("Opening shared memory device %s\n", memdevice);
 	if ((memfd = open(memdevice, O_RDWR)) < 0) {
-		fprintf(stderr, "%s: failed to open memory device %s, errno=%d\n", progname, memdevice, errno);
+		fprintf(stderr, "%s: failed to open memory device %s, "
+			"errno=%d\n", progname, memdevice, errno);
 		return(-1);
 	}
 
@@ -203,14 +202,16 @@ int download()
 	if (verbose)
 		printf("Stoping any current slave\n");
 	if (ioctl(memfd, STL_BSTOP, 0) < 0) {
-		fprintf(stderr, "%s: ioctl(STL_BSTOP) failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: ioctl(STL_BSTOP) failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 
 	if (verbose)
 		printf("Reseting the board\n");
 	if (ioctl(memfd, STL_BRESET, 0) < 0) {
-		fprintf(stderr, "%s: ioctl(STL_BRESET) failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: ioctl(STL_BRESET) failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 	if (reset)
@@ -223,7 +224,8 @@ int download()
 	if (verbose)
 		printf("Interrupting board to activate shared memory\n");
 	if (ioctl(memfd, STL_BINTR, 0) < 0) {
-		fprintf(stderr, "%s: ioctl(STL_BINTR) failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: ioctl(STL_BINTR) failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 	/*sleep(1);*/
@@ -231,7 +233,8 @@ int download()
 	if (verbose)
 		printf("Opening slave image file %s\n", image);
 	if ((ifd = open(image, O_RDONLY)) < 0) {
-		fprintf(stderr, "%s: failed to open image file %s, errno=%d\n", progname, image, errno);
+		fprintf(stderr, "%s: failed to open image file %s, errno=%d\n",
+			progname, image, errno);
 		return(-1);
 	}
 
@@ -245,11 +248,13 @@ int download()
 		printf("Reading ROM signature from board\n");
 
 	if (lseek(memfd, CDK_SIGADDR, SEEK_SET) != CDK_SIGADDR) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory file, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory file, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 	if (read(memfd, &ecpsig, sizeof(cdkecpsig_t)) < 0) {
-		fprintf(stderr, "%s: read of ROM signature failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: read of ROM signature failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 	if (ecpsig.magic == ECP_MAGIC) {
@@ -260,15 +265,17 @@ int download()
 	}
 
 	if (lseek(memfd, CDK_SIGADDR, SEEK_SET) != CDK_SIGADDR) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory file, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory file, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 	if (read(memfd, &onbsig, sizeof(cdkonbsig_t)) < 0) {
-		fprintf(stderr, "%s: read of ROM signature failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: read of ROM signature failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 	if ((onbsig.magic0 == ONB_MAGIC0) && (onbsig.magic1 == ONB_MAGIC1) &&
-			(onbsig.magic2 == ONB_MAGIC2) && (onbsig.magic3 == ONB_MAGIC3)) {
+	    (onbsig.magic2 == ONB_MAGIC2) && (onbsig.magic3 == ONB_MAGIC3)) {
 		nrdevs = onbfindports(&onbsig);
 		if (nrdevs < 0)
 			return(-1);
@@ -297,24 +304,29 @@ int download()
 	if (verbose)
 		printf("Copying vector table into shared memory\n");
 	if ((n = read(ifd, buf, CDK_SIGADDR)) < 0) {
-		fprintf(stderr, "%s: read of image file failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: read of image file failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 	if (lseek(memfd, 0, SEEK_SET) != 0) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory file, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory file, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 	if (write(memfd, buf, n) < 0) {
-		fprintf(stderr, "%s: write to memory device failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: write to memory device failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 
 	if (lseek(ifd, 0x1000, SEEK_SET) != 0x1000) {
-		fprintf(stderr, "%s: lseek(%x) failed on image file, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on image file, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 	if (lseek(memfd, 0x1000, SEEK_SET) != 0x1000) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory device, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory device, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 
@@ -323,11 +335,13 @@ int download()
  */
 	do {
 		if ((n = read(ifd, buf, BUFSIZE)) < 0) {
-			fprintf(stderr, "%s: read of image file failed, errno=%d\n", progname, errno);
+			fprintf(stderr, "%s: read of image file failed, "
+				"errno=%d\n", progname, errno);
 			return(-1);
 		}
 		if (write(memfd, buf, n) < 0) {
-			fprintf(stderr, "%s: write to memory device failed, errno=%d\n", progname, errno);
+			fprintf(stderr, "%s: write to memory device failed, "
+				"errno=%d\n", progname, errno);
 			return(-1);
 		}
 	} while (n == BUFSIZE);
@@ -344,11 +358,13 @@ int download()
 	if (verbose)
 		printf("Loading features into shared memory\n");
 	if (lseek(memfd, CDK_FEATADDR, SEEK_SET) != CDK_FEATADDR) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory device, errno=%d\n", progname, CDK_FEATADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory device, "
+			"errno=%d\n", progname, CDK_FEATADDR, errno);
 		return(-1);
 	}
 	if (write(memfd, &feature, sizeof(cdkfeature_t)) < 0) {
-		fprintf(stderr, "%s: write to memory device failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: write to memory device failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 
@@ -360,12 +376,14 @@ int download()
 	if (verbose)
 		printf("Setting alive marker to 0\n");
 	if (lseek(memfd, CDK_RDYADDR, SEEK_SET) != CDK_RDYADDR) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory device, errno=%d\n", progname, CDK_RDYADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory device, "
+			"errno=%d\n", progname, CDK_RDYADDR, errno);
 		return(-1);
 	}
 	alivemarker = 0;
 	if (write(memfd, &alivemarker, 1) < 0) {
-		fprintf(stderr, "%s: write to memory device failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: write to memory device failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 
@@ -376,20 +394,25 @@ int download()
 	if (verbose)
 		printf("Interrupting board to start slave image\n");
 	if (ioctl(memfd, STL_BINTR, 0) < 0) {
-		fprintf(stderr, "%s: ioctl(STL_BINTR) failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: ioctl(STL_BINTR) failed, errno=%d\n",
+			progname, errno);
 		return(-1);
 	}
 
 	strttime = time((time_t *) NULL);
 	if (verbose)
-		printf("Waiting for slave alive marker, time=%x timeout=%d\n", strttime, TIMEOUT);
+		printf("Waiting for slave alive marker, time=%x timeout=%d\n",
+			strttime, TIMEOUT);
 	while (time((time_t *) NULL) < (strttime + TIMEOUT)) {
 		if (lseek(memfd, CDK_RDYADDR, SEEK_SET) != CDK_RDYADDR) {
-			fprintf(stderr, "%s: lseek(%x) failed on memory device, errno=%d\n", progname, CDK_RDYADDR, errno);
+			fprintf(stderr, "%s: lseek(%x) failed on memory "
+				"device, errno=%d\n", progname, CDK_RDYADDR,
+				errno);
 			return(-1);
 		}
 		if (read(memfd, &alivemarker, 1) < 0){
-			fprintf(stderr, "%s: read of image file failed, errno=%d\n", progname, errno);
+			fprintf(stderr, "%s: read of image file failed, "
+				"errno=%d\n", progname, errno);
 			return(-1);
 		}
 		if (alivemarker == CDK_ALIVEMARKER)
@@ -402,12 +425,14 @@ int download()
 	}
 
 	if (lseek(memfd, CDK_RDYADDR, SEEK_SET) != CDK_RDYADDR) {
-		fprintf(stderr, "%s: lseek(%x) failed on memory device, errno=%d\n", progname, CDK_RDYADDR, errno);
+		fprintf(stderr, "%s: lseek(%x) failed on memory device, "
+			"errno=%d\n", progname, CDK_RDYADDR, errno);
 		return(-1);
 	}
 	alivemarker = 0;
 	if (write(memfd, &alivemarker, 1) < 0) {
-		fprintf(stderr, "%s: write to memory device failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: write to memory device failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 
@@ -422,7 +447,8 @@ int download()
 	if (verbose)
 		printf("Driver initializing host shared memory interface\n");
 	if (ioctl(memfd, STL_BSTART, 0) < 0) {
-		fprintf(stderr, "%s: ioctl(STL_BSTART) failed, errno=%d\n", progname, errno);
+		fprintf(stderr, "%s: ioctl(STL_BSTART) failed, "
+			"errno=%d\n", progname, errno);
 		return(-1);
 	}
 
@@ -482,7 +508,8 @@ void main(int argc, char *argv[])
 
 	if (memdevice == (char *) NULL) {
 		if ((brdnr < 0) || (brdnr >= 8)) {
-			fprintf(stderr, "%s: invalid board number %d specified\n", progname, brdnr);
+			fprintf(stderr, "%s: invalid board number %d "
+				"specified\n", progname, brdnr);
 			exit(1);
 		}
 		sprintf(devstr, defdevice, brdnr);
@@ -498,16 +525,19 @@ void main(int argc, char *argv[])
  *	Check that the shared memory device exits and is a character device.
  */
 	if (stat(memdevice, &statinfo) < 0) {
-		fprintf(stderr, "%s: memory device %s does not exist\n", progname, memdevice);
+		fprintf(stderr, "%s: memory device %s does not exist\n",
+			progname, memdevice);
 		exit(1);
 	}
 	if ((statinfo.st_mode & S_IFMT) != S_IFCHR) {
-		fprintf(stderr, "%s: memory device %s is not a char device\n", progname, memdevice);
+		fprintf(stderr, "%s: memory device %s is not a char device\n",
+			progname, memdevice);
 		exit(1);
 	}
 
 	if (stat(image, &statinfo) < 0) {
-		fprintf(stderr, "%s: image file %s does not exist\n", progname, image);
+		fprintf(stderr, "%s: image file %s does not exist\n",
+			progname, image);
 		exit(1);
 	}
 
